@@ -10,3 +10,9 @@ def run_preproc_clinvar(data):
     from .preproc_clinvar import PreprocClinVar
     pp = PreprocClinVar(data)
     pp.run()
+
+
+def run_preproc_gnomad(data):
+    from .preproc_gnomad import PreprocGnomAD
+    pp = PreprocGnomAD(data)
+    pp.run()
